﻿using ApiWithMessagePack.Common;
using ApiWithMessagePack.Models;
using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Formatting;
using System.Threading.Tasks;

namespace ApiWithMessagePack.Client
{
    public class WeatherForecastClient
    {
        private readonly HttpClient _httpClient;

        public WeatherForecastClient(HttpClient httpClient)
        {
            _httpClient = httpClient;
        }

        public async Task<WeatherForecast[]> GetAsync()
        {
            try
            {
                HttpRequestMessage requestMessage = new HttpRequestMessage(HttpMethod.Get, "WeatherForecast");

                requestMessage.Headers.Add("Accept", Constants.MediaTypes.Output.WeatherMPack);

                HttpResponseMessage response = await _httpClient.SendAsync(requestMessage).ConfigureAwait(false);

                if (response.IsSuccessStatusCode)
                {
                    var mediaTypeFormatter = new MediaTypeFormatter[]
                    {
                        new MessagePackMediaTypeFormatter(Constants.MediaTypes.Output.WeatherMPack)
                    };

                    WeatherForecast[] forecasts = await response
                        .Content
                        .ReadAsAsync<WeatherForecast[]>(mediaTypeFormatter)
                        .ConfigureAwait(false);

                    return forecasts;
                    
                }
                else if (response.StatusCode == System.Net.HttpStatusCode.InternalServerError)
                {
                    string content = await response.Content.ReadAsStringAsync().ConfigureAwait(false);

                    if (!string.IsNullOrEmpty(content))
                    {
                        throw new Exception(content);
                    }
                    else
                    {
                        throw new Exception(response.ReasonPhrase);
                    }
                }
                else
                {
                    //TODO :Return a response instead of the object
                    throw new Exception(response.ReasonPhrase);
                }
            }
            catch (Exception)
            {
                //LOG
                throw;
            }
        }
    }
}
