﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;

namespace ApiWithMessagePack.Common
{
    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false, Inherited = false)]
    public class AcceptStrictAttribute : Attribute, IActionFilter
    {
        private const string AcceptKey = "Accept";

        public AcceptStrictAttribute(params string[] acceptValues)
        {
            AcceptValues = acceptValues;
        }

        public string[] AcceptValues { get; }

        public void OnActionExecuted(ActionExecutedContext context)
        {
        }

        public void OnActionExecuting(ActionExecutingContext context)
        {
            if (!context.HttpContext.Request.Headers.ContainsKey(AcceptKey))
            {

                context.Result = new ObjectResult("The request does not contain the header Accept.")
                {
                    StatusCode = (int)HttpStatusCode.NotAcceptable
                };

                return;
            }

            List<string> values = context.HttpContext.Request.Headers[AcceptKey].ToList();

            bool found = false;

            foreach (string acceptV in AcceptValues)
            {
                if (values.Contains(acceptV))
                {
                    found = true;
                    break;
                }
            }

            if (!found)
            {
                context.Result = new ObjectResult($"Could not find a valid Accept Header value. Valid : [{string.Join(",", AcceptValues)}]")
                {
                    StatusCode = (int)HttpStatusCode.NotAcceptable
                };
            }
        }
    }
}
